@@ -45,9 +45,9 @@ struct rpl_genl_family {
 
 #define genl_family rpl_genl_family
 #define genl_notify rpl_genl_notify
-void genl_notify(struct genl_family *family,
-		 struct sk_buff *skb, struct net *net, u32 portid, u32 group,
-		 struct nlmsghdr *nlh, gfp_t flags);
+void rpl_genl_notify(struct genl_family *family,
+		     struct sk_buff *skb, struct net *net, u32 portid, u32 group,
+		     struct nlmsghdr *nlh, gfp_t flags);
 
 static inline void *rpl_genlmsg_put(struct sk_buff *skb, u32 portid, u32 seq,
 				    struct genl_family *family, int flags, u8 cmd)
@@ -67,7 +67,7 @@ static inline int rpl_genl_unregister_family(struct genl_family *family)
 static inline int genl_set_err(struct genl_family *family, struct net *net,
 			       u32 portid, u32 group, int code)
 {
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,33)
+#ifdef HAVE_VOID_NETLINK_SET_ERR
 	netlink_set_err(net->genl_sock, portid, group, code);
 	return 0;
 #else
@@ -105,4 +105,44 @@ static inline struct sk_buff *genlmsg_new_unicast(size_t payload,
 }
 #endif
 
+#ifndef HAVE_GENL_HAS_LISTENERS
+static inline int genl_has_listeners(struct genl_family *family,
+				     struct net *net, unsigned int group)
+{
+#ifdef HAVE_MCGRP_OFFSET
+	if (WARN_ON_ONCE(group >= family->n_mcgrps))
+		return -EINVAL;
+	group = family->mcgrp_offset + group;
+#endif
+	return netlink_has_listeners(net->genl_sock, group);
+}
+#else
+
+#ifndef HAVE_GENL_HAS_LISTENERS_TAKES_NET
+static inline int rpl_genl_has_listeners(struct genl_family *family,
+				         struct net *net, unsigned int group)
+{
+#ifdef HAVE_GENL_NOTIFY_TAKES_FAMILY
+    return genl_has_listeners(family, net->genl_sock, group);
+#else
+    return genl_has_listeners(&family->compat_family, net->genl_sock, group);
+#endif
+}
+
+#define genl_has_listeners rpl_genl_has_listeners
+#endif
+
+#endif /* HAVE_GENL_HAS_LISTENERS */
+
+#ifndef HAVE_GENLMSG_PARSE
+static inline int genlmsg_parse(const struct nlmsghdr *nlh,
+				const struct genl_family *family,
+				struct nlattr *tb[], int maxtype,
+				const struct nla_policy *policy)
+{
+	return nlmsg_parse(nlh, family->hdrsize + GENL_HDRLEN, tb, maxtype,
+			   policy);
+}
+#endif
+
 #endif /* genetlink.h */
