@@ -1,10 +1,10 @@
 #ifndef __NET_UDP_WRAPPER_H
 #define __NET_UDP_WRAPPER_H  1
 
-#include <linux/version.h>
+#include <net/ip.h>
 
 #ifdef inet_get_local_port_range
-/* RHEL7 backports udp_flow_src_port() using an older version of
+/* Earlier RHEL7 kernels backport udp_flow_src_port() using an older version of
  * inet_get_local_port_range(). */
 #undef inet_get_local_port_range
 #include_next <net/udp.h>
@@ -15,7 +15,7 @@
 
 #ifndef HAVE_UDP_FLOW_SRC_PORT
 static inline __be16 rpl_udp_flow_src_port(struct net *net, struct sk_buff *skb,
-					   int min, int max, bool use_eth)
+                                           int min, int max, bool use_eth)
 {
 	u32 hash;
 
@@ -46,4 +46,18 @@ static inline __be16 rpl_udp_flow_src_port(struct net *net, struct sk_buff *skb,
 #define udp_flow_src_port rpl_udp_flow_src_port
 #endif
 
+#ifndef HAVE_UDP_V4_CHECK
+static inline __sum16 udp_v4_check(int len, __be32 saddr,
+				   __be32 daddr, __wsum base)
+{
+	return csum_tcpudp_magic(saddr, daddr, len, IPPROTO_UDP, base);
+}
+#endif
+
+#ifndef HAVE_UDP_SET_CSUM
+#define udp_set_csum rpl_udp_set_csum
+void rpl_udp_set_csum(bool nocheck, struct sk_buff *skb,
+		      __be32 saddr, __be32 daddr, int len);
+#endif
+
 #endif
