@@ -21,26 +21,29 @@
 #include <stdbool.h>
 #include <inttypes.h>
 
+#include "packets.h"
+
 struct bfd;
 struct dpif_flow_stats;
 struct flow;
 struct flow_wildcards;
 struct netdev;
-struct ofpbuf;
+struct dp_packet;
 struct smap;
 
-void bfd_wait(const struct bfd *);
+long long int bfd_wait(const struct bfd *);
 void bfd_run(struct bfd *);
 
 bool bfd_should_send_packet(const struct bfd *);
-void bfd_put_packet(struct bfd *bfd, struct ofpbuf *packet,
-                    uint8_t eth_src[6]);
+void bfd_put_packet(struct bfd *bfd, struct dp_packet *packet,
+                    const struct eth_addr eth_src);
 
 bool bfd_should_process_flow(const struct bfd *, const struct flow *,
                              struct flow_wildcards *);
 void bfd_process_packet(struct bfd *, const struct flow *,
-                        const struct ofpbuf *);
+                        const struct dp_packet *);
 
+void bfd_init(void);
 struct bfd *bfd_configure(struct bfd *, const char *name,
                           const struct smap *smap,
                           struct netdev *netdev);
@@ -49,6 +52,7 @@ void bfd_unref(struct bfd *);
 
 void bfd_account_rx(struct bfd *, const struct dpif_flow_stats *);
 bool bfd_forwarding(struct bfd *);
+bool bfd_check_status_change(struct bfd *);
 void bfd_get_status(const struct bfd *, struct smap *);
 void bfd_set_netdev(struct bfd *, const struct netdev *);
 long long int bfd_wake_time(const struct bfd *);
