@@ -1,4 +1,4 @@
-/* Copyright (c) 2011, 2012, 2013 Nicira, Inc.
+/* Copyright (c) 2011, 2012, 2013, 2014, 2015 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -29,9 +29,7 @@
 #include "ofp-errors.h"
 #include "ofp-util.h"
 #include "openflow/nicira-ext.h"
-#include "vlog.h"
-
-#define BUNDLE_MAX_SLAVES 2048
+#include "openvswitch/vlog.h"
 
 VLOG_DEFINE_THIS_MODULE(bundle);
 
@@ -103,89 +101,6 @@ bundle_execute(const struct ofpact_bundle *bundle,
     }
 }
 
-/* Checks that 'nab' specifies a bundle action which is supported by this
- * bundle module.  Uses the 'max_ports' parameter to validate each port using
- * ofputil_check_output_port().  Returns 0 if 'nab' is supported, otherwise an
- * OFPERR_* error code. */
-enum ofperr
-bundle_from_openflow(const struct nx_action_bundle *nab,
-                     struct ofpbuf *ofpacts)
-{
-    static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
-    struct ofpact_bundle *bundle;
-    uint16_t subtype;
-    uint32_t slave_type;
-    size_t slaves_size, i;
-    enum ofperr error;
-
-    bundle = ofpact_put_BUNDLE(ofpacts);
-
-    subtype = ntohs(nab->subtype);
-    bundle->n_slaves = ntohs(nab->n_slaves);
-    bundle->basis = ntohs(nab->basis);
-    bundle->fields = ntohs(nab->fields);
-    bundle->algorithm = ntohs(nab->algorithm);
-    slave_type = ntohl(nab->slave_type);
-    slaves_size = ntohs(nab->len) - sizeof *nab;
-
-    error = OFPERR_OFPBAC_BAD_ARGUMENT;
-    if (!flow_hash_fields_valid(bundle->fields)) {
-        VLOG_WARN_RL(&rl, "unsupported fields %d", (int) bundle->fields);
-    } else if (bundle->n_slaves > BUNDLE_MAX_SLAVES) {
-        VLOG_WARN_RL(&rl, "too may slaves");
-    } else if (bundle->algorithm != NX_BD_ALG_HRW
-               && bundle->algorithm != NX_BD_ALG_ACTIVE_BACKUP) {
-        VLOG_WARN_RL(&rl, "unsupported algorithm %d", (int) bundle->algorithm);
-    } else if (slave_type != NXM_OF_IN_PORT) {
-        VLOG_WARN_RL(&rl, "unsupported slave type %"PRIu16, slave_type);
-    } else {
-        error = 0;
-    }
-
-    if (!is_all_zeros(nab->zero, sizeof nab->zero)) {
-        VLOG_WARN_RL(&rl, "reserved field is nonzero");
-        error = OFPERR_OFPBAC_BAD_ARGUMENT;
-    }
-
-    if (subtype == NXAST_BUNDLE && (nab->ofs_nbits || nab->dst)) {
-        VLOG_WARN_RL(&rl, "bundle action has nonzero reserved fields");
-        error = OFPERR_OFPBAC_BAD_ARGUMENT;
-    }
-
-    if (subtype == NXAST_BUNDLE_LOAD) {
-        bundle->dst.field = mf_from_nxm_header(ntohl(nab->dst));
-        bundle->dst.ofs = nxm_decode_ofs(nab->ofs_nbits);
-        bundle->dst.n_bits = nxm_decode_n_bits(nab->ofs_nbits);
-
-        if (bundle->dst.n_bits < 16) {
-            VLOG_WARN_RL(&rl, "bundle_load action requires at least 16 bit "
-                         "destination.");
-            error = OFPERR_OFPBAC_BAD_ARGUMENT;
-        }
-    }
-
-    if (slaves_size < bundle->n_slaves * sizeof(ovs_be16)) {
-        VLOG_WARN_RL(&rl, "Nicira action %"PRIu16" only has %"PRIuSIZE" bytes "
-                     "allocated for slaves.  %"PRIuSIZE" bytes are required for "
-                     "%"PRIu16" slaves.", subtype, slaves_size,
-                     bundle->n_slaves * sizeof(ovs_be16), bundle->n_slaves);
-        error = OFPERR_OFPBAC_BAD_LEN;
-    }
-
-    for (i = 0; i < bundle->n_slaves; i++) {
-        uint16_t ofp_port = ntohs(((ovs_be16 *)(nab + 1))[i]);
-        ofpbuf_put(ofpacts, &ofp_port, sizeof ofp_port);
-    }
-
-    bundle = ofpacts->l2;
-    ofpact_update_len(ofpacts, &bundle->ofpact);
-
-    if (!error) {
-        error = bundle_check(bundle, OFPP_MAX, NULL);
-    }
-    return error;
-}
-
 enum ofperr
 bundle_check(const struct ofpact_bundle *bundle, ofp_port_t max_ports,
              const struct flow *flow)
@@ -202,14 +117,14 @@ bundle_check(const struct ofpact_bundle *bundle, ofp_port_t max_ports,
 
     for (i = 0; i < bundle->n_slaves; i++) {
         ofp_port_t ofp_port = bundle->slaves[i];
-        enum ofperr error;
 
-        error = ofpact_check_output_port(ofp_port, max_ports);
-        if (error) {
-            VLOG_WARN_RL(&rl, "invalid slave %"PRIu16, ofp_port);
-            return error;
+        if (ofp_port != OFPP_NONE) {
+            enum ofperr error = ofpact_check_output_port(ofp_port, max_ports);
+            if (error) {
+                VLOG_WARN_RL(&rl, "invalid slave %"PRIu16, ofp_port);
+                return error;
+            }
         }
-
         /* Controller slaves are unsupported due to the lack of a max_len
          * argument. This may or may not change in the future.  There doesn't
          * seem to be a real-world use-case for supporting it. */
@@ -222,40 +137,12 @@ bundle_check(const struct ofpact_bundle *bundle, ofp_port_t max_ports,
     return 0;
 }
 
-void
-bundle_to_nxast(const struct ofpact_bundle *bundle, struct ofpbuf *openflow)
-{
-    int slaves_len = ROUND_UP(2 * bundle->n_slaves, OFP_ACTION_ALIGN);
-    struct nx_action_bundle *nab;
-    ovs_be16 *slaves;
-    size_t i;
-
-    nab = (bundle->dst.field
-           ? ofputil_put_NXAST_BUNDLE_LOAD(openflow)
-           : ofputil_put_NXAST_BUNDLE(openflow));
-    nab->len = htons(ntohs(nab->len) + slaves_len);
-    nab->algorithm = htons(bundle->algorithm);
-    nab->fields = htons(bundle->fields);
-    nab->basis = htons(bundle->basis);
-    nab->slave_type = htonl(NXM_OF_IN_PORT);
-    nab->n_slaves = htons(bundle->n_slaves);
-    if (bundle->dst.field) {
-        nab->ofs_nbits = nxm_encode_ofs_nbits(bundle->dst.ofs,
-                                              bundle->dst.n_bits);
-        nab->dst = htonl(bundle->dst.field->nxm_header);
-    }
-
-    slaves = ofpbuf_put_zeros(openflow, slaves_len);
-    for (i = 0; i < bundle->n_slaves; i++) {
-        slaves[i] = htons(ofp_to_u16(bundle->slaves[i]));
-    }
-}
 
 /* Helper for bundle_parse and bundle_parse_load.
  *
  * Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string.*/
-static char * WARN_UNUSED_RESULT
+static char * OVS_WARN_UNUSED_RESULT
 bundle_parse__(const char *s, char **save_ptr,
                const char *fields, const char *basis, const char *algorithm,
                const char *slave_type, const char *dst,
@@ -288,7 +175,7 @@ bundle_parse__(const char *s, char **save_ptr,
         }
         ofpbuf_put(ofpacts, &slave_port, sizeof slave_port);
 
-        bundle = ofpacts->l2;
+        bundle = ofpacts->header;
         bundle->n_slaves++;
     }
     ofpact_update_len(ofpacts, &bundle->ofpact);
@@ -299,6 +186,10 @@ bundle_parse__(const char *s, char **save_ptr,
         bundle->fields = NX_HASH_FIELDS_ETH_SRC;
     } else if (!strcasecmp(fields, "symmetric_l4")) {
         bundle->fields = NX_HASH_FIELDS_SYMMETRIC_L4;
+    } else if (!strcasecmp(fields, "symmetric_l3l4")) {
+        bundle->fields = NX_HASH_FIELDS_SYMMETRIC_L3L4;
+    } else if (!strcasecmp(fields, "symmetric_l3l4+udp")) {
+        bundle->fields = NX_HASH_FIELDS_SYMMETRIC_L3L4_UDP;
     } else {
         return xasprintf("%s: unknown fields `%s'", s, fields);
     }
@@ -320,6 +211,11 @@ bundle_parse__(const char *s, char **save_ptr,
         if (error) {
             return error;
         }
+
+        if (!mf_nxm_header(bundle->dst.field->id)) {
+            return xasprintf("%s: experimenter OXM field '%s' not supported",
+                             s, dst);
+        }
     }
 
     return NULL;
@@ -330,7 +226,7 @@ bundle_parse__(const char *s, char **save_ptr,
  *
  * Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string. */
-char * WARN_UNUSED_RESULT
+char * OVS_WARN_UNUSED_RESULT
 bundle_parse(const char *s, struct ofpbuf *ofpacts)
 {
     char *fields, *basis, *algorithm, *slave_type, *slave_delim;
@@ -357,7 +253,7 @@ bundle_parse(const char *s, struct ofpbuf *ofpacts)
  *
  * Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string.*/
-char * WARN_UNUSED_RESULT
+char * OVS_WARN_UNUSED_RESULT
 bundle_parse_load(const char *s, struct ofpbuf *ofpacts)
 {
     char *fields, *basis, *algorithm, *slave_type, *dst, *slave_delim;
