@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012, 2013 Nicira, Inc.
+ * Copyright (c) 2011, 2012, 2013, 2014, 2015 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,151 +30,6 @@
 #include "openflow/openflow.h"
 #include "unaligned.h"
 
-static ovs_be16
-get_be16(const void **pp)
-{
-    const ovs_be16 *p = *pp;
-    ovs_be16 value = *p;
-    *pp = p + 1;
-    return value;
-}
-
-static ovs_be32
-get_be32(const void **pp)
-{
-    const ovs_be32 *p = *pp;
-    ovs_be32 value = get_unaligned_be32(p);
-    *pp = p + 1;
-    return value;
-}
-
-static void
-get_subfield(int n_bits, const void **p, struct mf_subfield *sf)
-{
-    sf->field = mf_from_nxm_header(ntohl(get_be32(p)));
-    sf->ofs = ntohs(get_be16(p));
-    sf->n_bits = n_bits;
-}
-
-static unsigned int
-learn_min_len(uint16_t header)
-{
-    int n_bits = header & NX_LEARN_N_BITS_MASK;
-    int src_type = header & NX_LEARN_SRC_MASK;
-    int dst_type = header & NX_LEARN_DST_MASK;
-    unsigned int min_len;
-
-    min_len = 0;
-    if (src_type == NX_LEARN_SRC_FIELD) {
-        min_len += sizeof(ovs_be32); /* src_field */
-        min_len += sizeof(ovs_be16); /* src_ofs */
-    } else {
-        min_len += DIV_ROUND_UP(n_bits, 16);
-    }
-    if (dst_type == NX_LEARN_DST_MATCH ||
-        dst_type == NX_LEARN_DST_LOAD) {
-        min_len += sizeof(ovs_be32); /* dst_field */
-        min_len += sizeof(ovs_be16); /* dst_ofs */
-    }
-    return min_len;
-}
-
-/* Converts 'nal' into a "struct ofpact_learn" and appends that struct to
- * 'ofpacts'.  Returns 0 if successful, otherwise an OFPERR_*. */
-enum ofperr
-learn_from_openflow(const struct nx_action_learn *nal, struct ofpbuf *ofpacts)
-{
-    struct ofpact_learn *learn;
-    const void *p, *end;
-
-    if (nal->pad) {
-        return OFPERR_OFPBAC_BAD_ARGUMENT;
-    }
-
-    learn = ofpact_put_LEARN(ofpacts);
-
-    learn->idle_timeout = ntohs(nal->idle_timeout);
-    learn->hard_timeout = ntohs(nal->hard_timeout);
-    learn->priority = ntohs(nal->priority);
-    learn->cookie = ntohll(nal->cookie);
-    learn->table_id = nal->table_id;
-    learn->fin_idle_timeout = ntohs(nal->fin_idle_timeout);
-    learn->fin_hard_timeout = ntohs(nal->fin_hard_timeout);
-
-    /* We only support "send-flow-removed" for now. */
-    switch (ntohs(nal->flags)) {
-    case 0:
-        learn->flags = 0;
-        break;
-    case OFPFF_SEND_FLOW_REM:
-        learn->flags = OFPUTIL_FF_SEND_FLOW_REM;
-        break;
-    default:
-        return OFPERR_OFPBAC_BAD_ARGUMENT;
-    }
-
-    if (learn->table_id == 0xff) {
-        return OFPERR_OFPBAC_BAD_ARGUMENT;
-    }
-
-    end = (char *) nal + ntohs(nal->len);
-    for (p = nal + 1; p != end; ) {
-        struct ofpact_learn_spec *spec;
-        uint16_t header = ntohs(get_be16(&p));
-
-        if (!header) {
-            break;
-        }
-
-        spec = ofpbuf_put_zeros(ofpacts, sizeof *spec);
-        learn = ofpacts->l2;
-        learn->n_specs++;
-
-        spec->src_type = header & NX_LEARN_SRC_MASK;
-        spec->dst_type = header & NX_LEARN_DST_MASK;
-        spec->n_bits = header & NX_LEARN_N_BITS_MASK;
-
-        /* Check for valid src and dst type combination. */
-        if (spec->dst_type == NX_LEARN_DST_MATCH ||
-            spec->dst_type == NX_LEARN_DST_LOAD ||
-            (spec->dst_type == NX_LEARN_DST_OUTPUT &&
-             spec->src_type == NX_LEARN_SRC_FIELD)) {
-            /* OK. */
-        } else {
-            return OFPERR_OFPBAC_BAD_ARGUMENT;
-        }
-
-        /* Check that the arguments don't overrun the end of the action. */
-        if ((char *) end - (char *) p < learn_min_len(header)) {
-            return OFPERR_OFPBAC_BAD_LEN;
-        }
-
-        /* Get the source. */
-        if (spec->src_type == NX_LEARN_SRC_FIELD) {
-            get_subfield(spec->n_bits, &p, &spec->src);
-        } else {
-            int p_bytes = 2 * DIV_ROUND_UP(spec->n_bits, 16);
-
-            bitwise_copy(p, p_bytes, 0,
-                         &spec->src_imm, sizeof spec->src_imm, 0,
-                         spec->n_bits);
-            p = (const uint8_t *) p + p_bytes;
-        }
-
-        /* Get the destination. */
-        if (spec->dst_type == NX_LEARN_DST_MATCH ||
-            spec->dst_type == NX_LEARN_DST_LOAD) {
-            get_subfield(spec->n_bits, &p, &spec->dst);
-        }
-    }
-    ofpact_update_len(ofpacts, &learn->ofpact);
-
-    if (!is_all_zeros(p, (char *) end - (char *) p)) {
-        return OFPERR_OFPBAC_BAD_ARGUMENT;
-    }
-
-    return 0;
-}
 
 /* Checks that 'learn' is a valid action on 'flow'.  Returns 0 if it is valid,
  * otherwise an OFPERR_*. */
@@ -222,79 +77,6 @@ learn_check(const struct ofpact_learn *learn, const struct flow *flow)
     return 0;
 }
 
-static void
-put_be16(struct ofpbuf *b, ovs_be16 x)
-{
-    ofpbuf_put(b, &x, sizeof x);
-}
-
-static void
-put_be32(struct ofpbuf *b, ovs_be32 x)
-{
-    ofpbuf_put(b, &x, sizeof x);
-}
-
-static void
-put_u16(struct ofpbuf *b, uint16_t x)
-{
-    put_be16(b, htons(x));
-}
-
-static void
-put_u32(struct ofpbuf *b, uint32_t x)
-{
-    put_be32(b, htonl(x));
-}
-
-/* Converts 'learn' into a "struct nx_action_learn" and appends that action to
- * 'ofpacts'. */
-void
-learn_to_nxast(const struct ofpact_learn *learn, struct ofpbuf *openflow)
-{
-    const struct ofpact_learn_spec *spec;
-    struct nx_action_learn *nal;
-    size_t start_ofs;
-
-    start_ofs = openflow->size;
-    nal = ofputil_put_NXAST_LEARN(openflow);
-    nal->idle_timeout = htons(learn->idle_timeout);
-    nal->hard_timeout = htons(learn->hard_timeout);
-    nal->fin_idle_timeout = htons(learn->fin_idle_timeout);
-    nal->fin_hard_timeout = htons(learn->fin_hard_timeout);
-    nal->priority = htons(learn->priority);
-    nal->cookie = htonll(learn->cookie);
-    nal->flags = htons(learn->flags);
-    nal->table_id = learn->table_id;
-
-    for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
-        put_u16(openflow, spec->n_bits | spec->dst_type | spec->src_type);
-
-        if (spec->src_type == NX_LEARN_SRC_FIELD) {
-            put_u32(openflow, spec->src.field->nxm_header);
-            put_u16(openflow, spec->src.ofs);
-        } else {
-            size_t n_dst_bytes = 2 * DIV_ROUND_UP(spec->n_bits, 16);
-            uint8_t *bits = ofpbuf_put_zeros(openflow, n_dst_bytes);
-            bitwise_copy(&spec->src_imm, sizeof spec->src_imm, 0,
-                         bits, n_dst_bytes, 0,
-                         spec->n_bits);
-        }
-
-        if (spec->dst_type == NX_LEARN_DST_MATCH ||
-            spec->dst_type == NX_LEARN_DST_LOAD) {
-            put_u32(openflow, spec->dst.field->nxm_header);
-            put_u16(openflow, spec->dst.ofs);
-        }
-    }
-
-    if ((openflow->size - start_ofs) % 8) {
-        ofpbuf_put_zeros(openflow, 8 - (openflow->size - start_ofs) % 8);
-    }
-
-    nal = ofpbuf_at_assert(openflow, start_ofs, sizeof *nal);
-    nal->len = htons(openflow->size - start_ofs);
-}
-
 /* Composes 'fm' so that executing it will implement 'learn' given that the
  * packet being processed has 'flow' as its flow.
  *
@@ -313,17 +95,22 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
     fm->priority = learn->priority;
     fm->cookie = htonll(0);
     fm->cookie_mask = htonll(0);
-    fm->new_cookie = htonll(learn->cookie);
+    fm->new_cookie = learn->cookie;
     fm->modify_cookie = fm->new_cookie != OVS_BE64_MAX;
     fm->table_id = learn->table_id;
     fm->command = OFPFC_MODIFY_STRICT;
     fm->idle_timeout = learn->idle_timeout;
     fm->hard_timeout = learn->hard_timeout;
+    fm->importance = 0;
     fm->buffer_id = UINT32_MAX;
     fm->out_port = OFPP_NONE;
-    fm->flags = learn->flags;
+    fm->flags = 0;
+    if (learn->flags & NX_LEARN_F_SEND_FLOW_REM) {
+        fm->flags |= OFPUTIL_FF_SEND_FLOW_REM;
+    }
     fm->ofpacts = NULL;
     fm->ofpacts_len = 0;
+    fm->delete_reason = OFPRR_DELETE;
 
     if (learn->fin_idle_timeout || learn->fin_hard_timeout) {
         struct ofpact_fin_timeout *oft;
@@ -334,8 +121,8 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
     }
 
     for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
+        struct ofpact_set_field *sf;
         union mf_subvalue value;
-        int chunk, ofs;
 
         if (spec->src_type == NX_LEARN_SRC_FIELD) {
             mf_read_subfield(&spec->src, flow, &value);
@@ -349,25 +136,19 @@ learn_execute(const struct ofpact_learn *learn, const struct flow *flow,
             break;
 
         case NX_LEARN_DST_LOAD:
-            for (ofs = 0; ofs < spec->n_bits; ofs += chunk) {
-                struct ofpact_reg_load *load;
-
-                chunk = MIN(spec->n_bits - ofs, 64);
-
-                load = ofpact_put_REG_LOAD(ofpacts);
-                load->dst.field = spec->dst.field;
-                load->dst.ofs = spec->dst.ofs + ofs;
-                load->dst.n_bits = chunk;
-                bitwise_copy(&value, sizeof value, ofs,
-                             &load->subvalue, sizeof load->subvalue, 0,
-                             chunk);
-            }
+            sf = ofpact_put_reg_load(ofpacts);
+            sf->field = spec->dst.field;
+            bitwise_copy(&value, sizeof value, 0,
+                         &sf->value, spec->dst.field->n_bytes, spec->dst.ofs,
+                         spec->n_bits);
+            bitwise_one(&sf->mask, spec->dst.field->n_bytes, spec->dst.ofs,
+                        spec->n_bits);
             break;
 
         case NX_LEARN_DST_OUTPUT:
             if (spec->n_bits <= 16
                 || is_all_zeros(value.u8, sizeof value - 2)) {
-                ofp_port_t port = u16_to_ofp(ntohs(value.be16[7]));
+                ofp_port_t port = u16_to_ofp(ntohll(value.integer));
 
                 if (ofp_to_u16(port) < ofp_to_u16(OFPP_MAX)
                     || port == OFPP_IN_PORT
@@ -404,32 +185,18 @@ learn_mask(const struct ofpact_learn *learn, struct flow_wildcards *wc)
 
 /* Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string. */
-static char * WARN_UNUSED_RESULT
+static char * OVS_WARN_UNUSED_RESULT
 learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec)
 {
     const char *full_s = s;
-    const char *arrow = strstr(s, "->");
     struct mf_subfield dst;
     union mf_subvalue imm;
     char *error;
+    int err;
 
-    memset(&imm, 0, sizeof imm);
-    if (s[0] == '0' && (s[1] == 'x' || s[1] == 'X') && arrow) {
-        const char *in = arrow - 1;
-        uint8_t *out = imm.u8 + sizeof imm.u8 - 1;
-        int n = arrow - (s + 2);
-        int i;
-
-        for (i = 0; i < n; i++) {
-            int hexit = hexit_value(in[-i]);
-            if (hexit < 0) {
-                return xasprintf("%s: bad hex digit in value", full_s);
-            }
-            out[-(i / 2)] |= i % 2 ? hexit << 4 : hexit;
-        }
-        s = arrow;
-    } else {
-        imm.be64[1] = htonll(strtoull(s, (char **) &s, 0));
+    err = parse_int_string(s, imm.u8, sizeof imm.u8, (char **) &s);
+    if (err) {
+        return xasprintf("%s: too many bits in immediate value", full_s);
     }
 
     if (strncmp(s, "->", 2)) {
@@ -441,6 +208,10 @@ learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec)
     if (error) {
         return error;
     }
+    if (!mf_nxm_header(dst.field->id)) {
+        return xasprintf("%s: experimenter OXM field '%s' not supported",
+                         full_s, s);
+    }
 
     if (!bitwise_is_all_zeros(&imm, sizeof imm, dst.n_bits,
                               (8 * sizeof imm) - dst.n_bits)) {
@@ -458,7 +229,7 @@ learn_parse_load_immediate(const char *s, struct ofpact_learn_spec *spec)
 
 /* Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string. */
-static char * WARN_UNUSED_RESULT
+static char * OVS_WARN_UNUSED_RESULT
 learn_parse_spec(const char *orig, char *name, char *value,
                  struct ofpact_learn_spec *spec)
 {
@@ -489,6 +260,10 @@ learn_parse_spec(const char *orig, char *name, char *value,
         if (error) {
             return error;
         }
+        if (!mf_nxm_header(spec->dst.field->id)) {
+            return xasprintf("%s: experimenter OXM field '%s' not supported",
+                             orig, name);
+        }
 
         /* Parse source and check prerequisites. */
         if (value[0] != '\0') {
@@ -547,7 +322,7 @@ learn_parse_spec(const char *orig, char *name, char *value,
 
 /* Returns NULL if successful, otherwise a malloc()'d string describing the
  * error.  The caller is responsible for freeing the returned string. */
-static char * WARN_UNUSED_RESULT
+static char * OVS_WARN_UNUSED_RESULT
 learn_parse__(char *orig, char *arg, struct ofpbuf *ofpacts)
 {
     struct ofpact_learn *learn;
@@ -579,13 +354,17 @@ learn_parse__(char *orig, char *arg, struct ofpbuf *ofpacts)
         } else if (!strcmp(name, "fin_hard_timeout")) {
             learn->fin_hard_timeout = atoi(value);
         } else if (!strcmp(name, "cookie")) {
-            learn->cookie = strtoull(value, NULL, 0);
+            learn->cookie = htonll(strtoull(value, NULL, 0));
+        } else if (!strcmp(name, "send_flow_rem")) {
+            learn->flags |= NX_LEARN_F_SEND_FLOW_REM;
+        } else if (!strcmp(name, "delete_learned")) {
+            learn->flags |= NX_LEARN_F_DELETE_LEARNED;
         } else {
             struct ofpact_learn_spec *spec;
             char *error;
 
             spec = ofpbuf_put_zeros(ofpacts, sizeof *spec);
-            learn = ofpacts->l2;
+            learn = ofpacts->header;
             learn->n_specs++;
 
             error = learn_parse_spec(orig, name, value, spec);
@@ -618,7 +397,7 @@ learn_parse__(char *orig, char *arg, struct ofpbuf *ofpacts)
  * the action's arguments.
  *
  * Modifies 'arg'. */
-char * WARN_UNUSED_RESULT
+char * OVS_WARN_UNUSED_RESULT
 learn_parse(char *arg, struct ofpbuf *ofpacts)
 {
     char *orig = xstrdup(arg);
@@ -653,11 +432,14 @@ learn_format(const struct ofpact_learn *learn, struct ds *s)
     if (learn->priority != OFP_DEFAULT_PRIORITY) {
         ds_put_format(s, ",priority=%"PRIu16, learn->priority);
     }
-    if (learn->flags & OFPFF_SEND_FLOW_REM) {
-        ds_put_cstr(s, ",OFPFF_SEND_FLOW_REM");
+    if (learn->flags & NX_LEARN_F_SEND_FLOW_REM) {
+        ds_put_cstr(s, ",send_flow_rem");
+    }
+    if (learn->flags & NX_LEARN_F_DELETE_LEARNED) {
+        ds_put_cstr(s, ",delete_learned");
     }
     if (learn->cookie != 0) {
-        ds_put_format(s, ",cookie=%#"PRIx64, learn->cookie);
+        ds_put_format(s, ",cookie=%#"PRIx64, ntohll(learn->cookie));
     }
 
     for (spec = learn->specs; spec < &learn->specs[learn->n_specs]; spec++) {
