@@ -373,8 +373,8 @@ nl_sock_subscribe_packets(struct nl_sock *sock)
 
     error = nl_sock_subscribe_packet__(sock, true);
     if (error) {
-        VLOG_WARN("could not unsubscribe packets (%s)",
-                  ovs_strerror(errno));
+        VLOG_WARN("could not subscribe packets (%s)",
+                  ovs_strerror(error));
         return error;
     }
     sock->read_ioctl = OVS_IOCTL_READ_PACKET;
@@ -389,8 +389,8 @@ nl_sock_unsubscribe_packets(struct nl_sock *sock)
 
     int error = nl_sock_subscribe_packet__(sock, false);
     if (error) {
-        VLOG_WARN("could not subscribe to packets (%s)",
-                  ovs_strerror(errno));
+        VLOG_WARN("could not unsubscribe to packets (%s)",
+                  ovs_strerror(error));
         return error;
     }
 
@@ -1179,10 +1179,17 @@ pend_io_request(struct nl_sock *sock)
     struct ovs_header *ovs_header;
     struct nlmsghdr *nlmsg;
     uint32_t seq;
-    int retval;
+    int retval = 0;
     int error;
     DWORD bytes;
     OVERLAPPED *overlapped = CONST_CAST(OVERLAPPED *, &sock->overlapped);
+    uint16_t cmd = OVS_CTRL_CMD_WIN_PEND_PACKET_REQ;
+
+    ovs_assert(sock->read_ioctl == OVS_IOCTL_READ_PACKET ||
+               sock->read_ioctl  == OVS_IOCTL_READ_EVENT);
+    if (sock->read_ioctl == OVS_IOCTL_READ_EVENT) {
+        cmd = OVS_CTRL_CMD_WIN_PEND_REQ;
+    }
 
     int ovs_msg_size = sizeof (struct nlmsghdr) + sizeof (struct genlmsghdr) +
                                sizeof (struct ovs_header);
@@ -1191,7 +1198,7 @@ pend_io_request(struct nl_sock *sock)
 
     seq = nl_sock_allocate_seq(sock, 1);
     nl_msg_put_genlmsghdr(&request, 0, OVS_WIN_NL_CTRL_FAMILY_ID, 0,
-                          OVS_CTRL_CMD_WIN_PEND_REQ, OVS_WIN_CONTROL_VERSION);
+                          cmd, OVS_WIN_CONTROL_VERSION);
     nlmsg = nl_msg_nlmsghdr(&request);
     nlmsg->nlmsg_seq = seq;
     nlmsg->nlmsg_pid = sock->pid;
@@ -1207,13 +1214,10 @@ pend_io_request(struct nl_sock *sock)
         if (error != ERROR_IO_INCOMPLETE && error != ERROR_IO_PENDING) {
             VLOG_ERR("nl_sock_wait failed - %s\n", ovs_format_message(error));
             retval = EINVAL;
-            goto done;
         }
     } else {
-        /* The I/O was completed synchronously */
-        poll_immediate_wake();
+        retval = EAGAIN;
     }
-    retval = 0;
 
 done:
     ofpbuf_uninit(&request);
@@ -1229,15 +1233,21 @@ nl_sock_wait(const struct nl_sock *sock, short int events)
 {
 #ifdef _WIN32
     if (sock->overlapped.Internal != STATUS_PENDING) {
-        pend_io_request(CONST_CAST(struct nl_sock *, sock));
-       /* XXX: poll_wevent_wait(sock->overlapped.hEvent); */
+        int ret = pend_io_request(CONST_CAST(struct nl_sock *, sock));
+        if (ret == 0) {
+            poll_wevent_wait(sock->overlapped.hEvent);
+        } else {
+            poll_immediate_wake();
+        }
+    } else {
+        poll_wevent_wait(sock->overlapped.hEvent);
     }
-    poll_immediate_wake(); /* XXX: temporary. */
 #else
     poll_fd_wait(sock->fd, events);
 #endif
 }
 
+#ifndef _WIN32
 /* Returns the underlying fd for 'sock', for use in "poll()"-like operations
  * that can't use nl_sock_wait().
  *
@@ -1248,13 +1258,9 @@ nl_sock_wait(const struct nl_sock *sock, short int events)
 int
 nl_sock_fd(const struct nl_sock *sock)
 {
-#ifdef _WIN32
-    BUILD_ASSERT_DECL(sizeof sock->handle == sizeof(int));
-    return (int)sock->handle;
-#else
     return sock->fd;
-#endif
 }
+#endif
 
 /* Returns the PID associated with this socket. */
 uint32_t
