@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012, 2013 Nicira, Inc.
+ * Copyright (c) 2012, 2013, 2014, 2015 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -42,7 +42,7 @@
 #include "ofp-errors.h"
 #include "util.h"
 
-struct list;
+struct ovs_list;
 
 /* Raw identifiers for OpenFlow messages.
  *
@@ -148,7 +148,7 @@ enum ofpraw {
     OFPRAW_OFPT11_PACKET_IN,
     /* OFPT 1.2 (10): struct ofp12_packet_in, uint8_t[]. */
     OFPRAW_OFPT12_PACKET_IN,
-    /* OFPT 1.3 (10): struct ofp13_packet_in, uint8_t[]. */
+    /* OFPT 1.3+ (10): struct ofp13_packet_in, uint8_t[]. */
     OFPRAW_OFPT13_PACKET_IN,
     /* NXT 1.0+ (17): struct nx_packet_in, uint8_t[]. */
     OFPRAW_NXT_PACKET_IN,
@@ -162,31 +162,39 @@ enum ofpraw {
 
     /* OFPT 1.0 (12): struct ofp_port_status, struct ofp10_phy_port. */
     OFPRAW_OFPT10_PORT_STATUS,
-    /* OFPT 1.1+ (12): struct ofp_port_status, struct ofp11_port. */
+    /* OFPT 1.1-1.3 (12): struct ofp_port_status, struct ofp11_port. */
     OFPRAW_OFPT11_PORT_STATUS,
+    /* OFPT 1.4+ (12): struct ofp_port_status, struct ofp14_port, uint8_t[8][]. */
+    OFPRAW_OFPT14_PORT_STATUS,
 
     /* OFPT 1.0 (13): struct ofp10_packet_out, uint8_t[]. */
     OFPRAW_OFPT10_PACKET_OUT,
     /* OFPT 1.1+ (13): struct ofp11_packet_out, uint8_t[]. */
     OFPRAW_OFPT11_PACKET_OUT,
 
-    /* OFPT 1.0 (14): struct ofp10_flow_mod, struct ofp_action_header[]. */
+    /* OFPT 1.0 (14): struct ofp10_flow_mod, uint8_t[8][]. */
     OFPRAW_OFPT10_FLOW_MOD,
     /* OFPT 1.1+ (14): struct ofp11_flow_mod, struct ofp11_instruction[]. */
     OFPRAW_OFPT11_FLOW_MOD,
     /* NXT 1.0+ (13): struct nx_flow_mod, uint8_t[8][]. */
     OFPRAW_NXT_FLOW_MOD,
 
-    /* OFPT 1.1+ (15): struct ofp11_group_mod, uint8_t[8][]. */
+    /* OFPT 1.1-1.4 (15): struct ofp11_group_mod, uint8_t[8][]. */
     OFPRAW_OFPT11_GROUP_MOD,
+    /* OFPT 1.5+ (15): struct ofp15_group_mod, uint8_t[8][]. */
+    OFPRAW_OFPT15_GROUP_MOD,
 
     /* OFPT 1.0 (15): struct ofp10_port_mod. */
     OFPRAW_OFPT10_PORT_MOD,
-    /* OFPT 1.1+ (16): struct ofp11_port_mod. */
+    /* OFPT 1.1-1.3 (16): struct ofp11_port_mod. */
     OFPRAW_OFPT11_PORT_MOD,
+    /* OFPT 1.4+ (16): struct ofp14_port_mod, uint8_t[8][]. */
+    OFPRAW_OFPT14_PORT_MOD,
 
-    /* OFPT 1.1+ (17): struct ofp11_table_mod. */
+    /* OFPT 1.1-1.3 (17): struct ofp11_table_mod. */
     OFPRAW_OFPT11_TABLE_MOD,
+    /* OFPT 1.4+ (17): struct ofp14_table_mod, uint8_t[8][]. */
+    OFPRAW_OFPT14_TABLE_MOD,
 
     /* OFPT 1.0 (18): void. */
     OFPRAW_OFPT10_BARRIER_REQUEST,
@@ -218,14 +226,20 @@ enum ofpraw {
     /* NXT 1.0+ (11): struct nx_role_request. */
     OFPRAW_NXT_ROLE_REPLY,
 
-    /* OFPT 1.3+ (26): void. */
+    /* OFPT 1.3 (26): void. */
     OFPRAW_OFPT13_GET_ASYNC_REQUEST,
-    /* OFPT 1.3+ (27): struct ofp13_async_config. */
+    /* OFPT 1.4+ (26): void. */
+    OFPRAW_OFPT14_GET_ASYNC_REQUEST,
+    /* OFPT 1.3 (27): struct ofp13_async_config. */
     OFPRAW_OFPT13_GET_ASYNC_REPLY,
-    /* OFPT 1.3+ (28): struct ofp13_async_config. */
+    /* OFPT 1.4+ (27): struct ofp14_async_config, uint8_t[8][]. */
+    OFPRAW_OFPT14_GET_ASYNC_REPLY,
+    /* OFPT 1.3 (28): struct ofp13_async_config. */
     OFPRAW_OFPT13_SET_ASYNC,
     /* NXT 1.0+ (19): struct nx_async_config. */
     OFPRAW_NXT_SET_ASYNC_CONFIG,
+    /* OFPT 1.4+ (28): struct ofp14_async_config, uint8_t[8][]. */
+    OFPRAW_OFPT14_SET_ASYNC,
 
     /* OFPT 1.3+ (29): struct ofp13_meter_mod, uint8_t[8][]. */
     OFPRAW_OFPT13_METER_MOD,
@@ -233,6 +247,15 @@ enum ofpraw {
     /* OFPT 1.4+ (30): struct ofp14_role_status, uint8_t[8][]. */
     OFPRAW_OFPT14_ROLE_STATUS,
 
+    /* OFPT 1.4+ (32): struct ofp14_requestforward, uint8_t[8][]. */
+    OFPRAW_OFPT14_REQUESTFORWARD,
+
+    /* OFPT 1.4+ (33): struct ofp14_bundle_ctrl_msg, uint8_t[8][]. */
+    OFPRAW_OFPT14_BUNDLE_CONTROL,
+
+    /* OFPT 1.4+ (34): struct ofp14_bundle_ctrl_msg, uint8_t[]. */
+    OFPRAW_OFPT14_BUNDLE_ADD_MESSAGE,
+
 /* Standard statistics. */
 
     /* OFPST 1.0+ (0): void. */
@@ -278,7 +301,7 @@ enum ofpraw {
     OFPRAW_OFPST11_TABLE_REPLY,
     /* OFPST 1.2 (3): struct ofp12_table_stats[]. */
     OFPRAW_OFPST12_TABLE_REPLY,
-    /* OFPST 1.3 (3): struct ofp13_table_stats[]. */
+    /* OFPST 1.3+ (3): struct ofp13_table_stats[]. */
     OFPRAW_OFPST13_TABLE_REPLY,
 
     /* OFPST 1.0 (4): struct ofp10_port_stats_request. */
@@ -290,8 +313,10 @@ enum ofpraw {
     OFPRAW_OFPST10_PORT_REPLY,
     /* OFPST 1.1-1.2 (4): struct ofp11_port_stats[]. */
     OFPRAW_OFPST11_PORT_REPLY,
-    /* OFPST 1.3+ (4): struct ofp13_port_stats[]. */
+    /* OFPST 1.3 (4): struct ofp13_port_stats[]. */
     OFPRAW_OFPST13_PORT_REPLY,
+    /* OFPST 1.4+ (4): uint8_t[8][]. */
+    OFPRAW_OFPST14_PORT_REPLY,
 
     /* OFPST 1.0 (5): struct ofp10_queue_stats_request. */
     OFPRAW_OFPST10_QUEUE_REQUEST,
@@ -302,19 +327,23 @@ enum ofpraw {
     OFPRAW_OFPST10_QUEUE_REPLY,
     /* OFPST 1.1-1.2 (5): struct ofp11_queue_stats[]. */
     OFPRAW_OFPST11_QUEUE_REPLY,
-    /* OFPST 1.3+ (5): struct ofp13_queue_stats[]. */
+    /* OFPST 1.3 (5): struct ofp13_queue_stats[]. */
     OFPRAW_OFPST13_QUEUE_REPLY,
+    /* OFPST 1.4+ (5): uint8_t[8][]. */
+    OFPRAW_OFPST14_QUEUE_REPLY,
 
     /* OFPST 1.1+ (6): struct ofp11_group_stats_request. */
     OFPRAW_OFPST11_GROUP_REQUEST,
 
     /* OFPST 1.1-1.2 (6): uint8_t[8][]. */
     OFPRAW_OFPST11_GROUP_REPLY,
-    /* OFPST 1.3 (6): uint8_t[8][]. */
+    /* OFPST 1.3+ (6): uint8_t[8][]. */
     OFPRAW_OFPST13_GROUP_REPLY,
 
-    /* OFPST 1.1+ (7): void. */
+    /* OFPST 1.1-1.4 (7): void. */
     OFPRAW_OFPST11_GROUP_DESC_REQUEST,
+    /* OFPST 1.5+ (7): struct ofp15_group_desc_request. */
+    OFPRAW_OFPST15_GROUP_DESC_REQUEST,
 
     /* OFPST 1.1+ (7): uint8_t[8][]. */
     OFPRAW_OFPST11_GROUP_DESC_REPLY,
@@ -349,13 +378,33 @@ enum ofpraw {
     /* OFPST 1.3+ (12): struct ofp13_table_features, uint8_t[8][]. */
     OFPRAW_OFPST13_TABLE_FEATURES_REPLY,
 
-    /* OFPST 1.0+ (13): void. */
-    OFPRAW_OFPST_PORT_DESC_REQUEST,
+    /* OFPST 1.4+ (14): void. */
+    OFPRAW_OFPST14_TABLE_DESC_REQUEST,
+
+    /* OFPST 1.4+ (14): struct ofp14_table_desc, uint8_t[8][]. */
+    OFPRAW_OFPST14_TABLE_DESC_REPLY,
+
+    /* OFPST 1.0-1.4 (13): void. */
+    OFPRAW_OFPST10_PORT_DESC_REQUEST,
+    /* OFPST 1.5+ (13): struct ofp15_port_desc_request. */
+    OFPRAW_OFPST15_PORT_DESC_REQUEST,
 
     /* OFPST 1.0 (13): struct ofp10_phy_port[]. */
     OFPRAW_OFPST10_PORT_DESC_REPLY,
-    /* OFPST 1.1+ (13): struct ofp11_port[]. */
+    /* OFPST 1.1-1.3 (13): struct ofp11_port[]. */
     OFPRAW_OFPST11_PORT_DESC_REPLY,
+    /* OFPST 1.4+ (13): uint8_t[8][]. */
+    OFPRAW_OFPST14_PORT_DESC_REPLY,
+
+    /* OFPST 1.4+ (16): uint8_t[8][]. */
+    OFPRAW_OFPST14_FLOW_MONITOR_REQUEST,
+    /* NXST 1.0 (2): uint8_t[8][]. */
+    OFPRAW_NXST_FLOW_MONITOR_REQUEST,
+
+    /* OFPST 1.4+ (16): uint8_t[8][]. */
+    OFPRAW_OFPST14_FLOW_MONITOR_REPLY,
+    /* NXST 1.0 (2): uint8_t[8][]. */
+    OFPRAW_NXST_FLOW_MONITOR_REPLY,
 
 /* Nicira extension messages.
  *
@@ -386,16 +435,14 @@ enum ofpraw {
     /* NXT 1.0+ (23): void. */
     OFPRAW_NXT_FLOW_MONITOR_RESUMED,
 
-/* Nicira extension statistics.
- *
- * Nicira extension statistics that correspond to standard OpenFlow statistics
- * are listed alongside the standard versions above. */
+    /* NXT 1.0+ (24): struct nx_tlv_table_mod, struct nx_tlv_map[]. */
+    OFPRAW_NXT_TLV_TABLE_MOD,
 
-    /* NXST 1.0 (2): uint8_t[8][]. */
-    OFPRAW_NXST_FLOW_MONITOR_REQUEST,
+    /* NXT 1.0+ (25): void. */
+    OFPRAW_NXT_TLV_TABLE_REQUEST,
 
-    /* NXST 1.0 (2): uint8_t[8][]. */
-    OFPRAW_NXST_FLOW_MONITOR_REPLY,
+    /* NXT 1.0+ (26): struct nx_tlv_table_reply, struct nx_tlv_map[]. */
+    OFPRAW_NXT_TLV_TABLE_REPLY,
 };
 
 /* Decoding messages into OFPRAW_* values. */
@@ -465,7 +512,8 @@ enum ofptype {
                                   * OFPRAW_OFPT11_FLOW_REMOVED.
                                   * OFPRAW_NXT_FLOW_REMOVED. */
     OFPTYPE_PORT_STATUS,         /* OFPRAW_OFPT10_PORT_STATUS.
-                                  * OFPRAW_OFPT11_PORT_STATUS. */
+                                  * OFPRAW_OFPT11_PORT_STATUS.
+                                  * OFPRAW_OFPT14_PORT_STATUS. */
 
     /* Controller command messages. */
     OFPTYPE_PACKET_OUT,          /* OFPRAW_OFPT10_PACKET_OUT.
@@ -473,10 +521,13 @@ enum ofptype {
     OFPTYPE_FLOW_MOD,            /* OFPRAW_OFPT10_FLOW_MOD.
                                   * OFPRAW_OFPT11_FLOW_MOD.
                                   * OFPRAW_NXT_FLOW_MOD. */
-    OFPTYPE_GROUP_MOD,           /* OFPRAW_OFPT11_GROUP_MOD. */
+    OFPTYPE_GROUP_MOD,           /* OFPRAW_OFPT11_GROUP_MOD.
+                                  * OFPRAW_OFPT15_GROUP_MOD. */
     OFPTYPE_PORT_MOD,            /* OFPRAW_OFPT10_PORT_MOD.
-                                  * OFPRAW_OFPT11_PORT_MOD. */
-    OFPTYPE_TABLE_MOD,           /* OFPRAW_OFPT11_TABLE_MOD. */
+                                  * OFPRAW_OFPT11_PORT_MOD.
+                                  * OFPRAW_OFPT14_PORT_MOD. */
+    OFPTYPE_TABLE_MOD,           /* OFPRAW_OFPT11_TABLE_MOD.
+                                  * OFPRAW_OFPT14_TABLE_MOD. */
 
     /* Barrier messages. */
     OFPTYPE_BARRIER_REQUEST,     /* OFPRAW_OFPT10_BARRIER_REQUEST.
@@ -497,10 +548,13 @@ enum ofptype {
                                    * OFPRAW_NXT_ROLE_REPLY. */
 
     /* Asynchronous message configuration. */
-    OFPTYPE_GET_ASYNC_REQUEST,    /* OFPRAW_OFPT13_GET_ASYNC_REQUEST. */
-    OFPTYPE_GET_ASYNC_REPLY,      /* OFPRAW_OFPT13_GET_ASYNC_REPLY. */
+    OFPTYPE_GET_ASYNC_REQUEST,    /* OFPRAW_OFPT13_GET_ASYNC_REQUEST.
+                                   * OFPRAW_OFPT14_GET_ASYNC_REQUEST. */
+    OFPTYPE_GET_ASYNC_REPLY,      /* OFPRAW_OFPT13_GET_ASYNC_REPLY.
+                                   * OFPRAW_OFPT14_GET_ASYNC_REPLY. */
     OFPTYPE_SET_ASYNC_CONFIG,     /* OFPRAW_NXT_SET_ASYNC_CONFIG.
-                                   * OFPRAW_OFPT13_SET_ASYNC. */
+                                   * OFPRAW_OFPT13_SET_ASYNC.
+                                   * OFPRAW_OFPT14_SET_ASYNC. */
 
     /* Meters and rate limiters configuration messages. */
     OFPTYPE_METER_MOD,            /* OFPRAW_OFPT13_METER_MOD. */
@@ -508,6 +562,13 @@ enum ofptype {
     /* Controller role change event messages. */
     OFPTYPE_ROLE_STATUS,          /* OFPRAW_OFPT14_ROLE_STATUS. */
 
+    /* Request forwarding by the switch. */
+    OFPTYPE_REQUESTFORWARD,       /* OFPRAW_OFPT14_REQUESTFORWARD. */
+
+    OFPTYPE_BUNDLE_CONTROL,       /* OFPRAW_OFPT14_BUNDLE_CONTROL. */
+
+    OFPTYPE_BUNDLE_ADD_MESSAGE,   /* OFPRAW_OFPT14_BUNDLE_ADD_MESSAGE. */
+
     /* Statistics. */
     OFPTYPE_DESC_STATS_REQUEST,      /* OFPRAW_OFPST_DESC_REQUEST. */
     OFPTYPE_DESC_STATS_REPLY,        /* OFPRAW_OFPST_DESC_REPLY. */
@@ -532,19 +593,22 @@ enum ofptype {
                                       * OFPRAW_OFPST11_PORT_REQUEST. */
     OFPTYPE_PORT_STATS_REPLY,        /* OFPRAW_OFPST10_PORT_REPLY.
                                       * OFPRAW_OFPST11_PORT_REPLY.
-                                      * OFPRAW_OFPST13_PORT_REPLY. */
+                                      * OFPRAW_OFPST13_PORT_REPLY.
+                                      * OFPRAW_OFPST14_PORT_REPLY. */
     OFPTYPE_QUEUE_STATS_REQUEST,     /* OFPRAW_OFPST10_QUEUE_REQUEST.
                                       * OFPRAW_OFPST11_QUEUE_REQUEST. */
     OFPTYPE_QUEUE_STATS_REPLY,       /* OFPRAW_OFPST10_QUEUE_REPLY.
                                       * OFPRAW_OFPST11_QUEUE_REPLY.
-                                      * OFPRAW_OFPST13_QUEUE_REPLY. */
+                                      * OFPRAW_OFPST13_QUEUE_REPLY.
+                                      * OFPRAW_OFPST14_QUEUE_REPLY. */
 
     OFPTYPE_GROUP_STATS_REQUEST,     /* OFPRAW_OFPST11_GROUP_REQUEST. */
 
     OFPTYPE_GROUP_STATS_REPLY,       /* OFPRAW_OFPST11_GROUP_REPLY.
                                       * OFPRAW_OFPST13_GROUP_REPLY. */
 
-    OFPTYPE_GROUP_DESC_STATS_REQUEST, /* OFPRAW_OFPST11_GROUP_DESC_REQUEST. */
+    OFPTYPE_GROUP_DESC_STATS_REQUEST, /* OFPRAW_OFPST11_GROUP_DESC_REQUEST.
+                                       * OFPRAW_OFPST15_GROUP_DESC_REQUEST. */
 
     OFPTYPE_GROUP_DESC_STATS_REPLY,  /* OFPRAW_OFPST11_GROUP_DESC_REPLY. */
 
@@ -568,10 +632,21 @@ enum ofptype {
 
     OFPTYPE_TABLE_FEATURES_STATS_REPLY, /* OFPRAW_OFPST13_TABLE_FEATURES_REPLY. */
 
-    OFPTYPE_PORT_DESC_STATS_REQUEST, /* OFPRAW_OFPST_PORT_DESC_REQUEST. */
+    OFPTYPE_TABLE_DESC_REQUEST,      /* OFPRAW_OFPST14_TABLE_DESC_REQUEST. */
+
+    OFPTYPE_TABLE_DESC_REPLY,        /* OFPRAW_OFPST14_TABLE_DESC_REPLY. */
+
+    OFPTYPE_PORT_DESC_STATS_REQUEST, /* OFPRAW_OFPST10_PORT_DESC_REQUEST.
+                                      * OFPRAW_OFPST15_PORT_DESC_REQUEST. */
 
     OFPTYPE_PORT_DESC_STATS_REPLY,   /* OFPRAW_OFPST10_PORT_DESC_REPLY.
-                                      * OFPRAW_OFPST11_PORT_DESC_REPLY. */
+                                      * OFPRAW_OFPST11_PORT_DESC_REPLY.
+                                      * OFPRAW_OFPST14_PORT_DESC_REPLY. */
+
+    OFPTYPE_FLOW_MONITOR_STATS_REQUEST, /* OFPRAW_OFPST14_FLOW_MONITOR_REQUEST.
+                                         * OFPRAW_NXST_FLOW_MONITOR_REQUEST. */
+    OFPTYPE_FLOW_MONITOR_STATS_REPLY,   /* OFPRAW_OFPST14_FLOW_MONITOR_REPLY.
+                                         * OFPRAW_NXST_FLOW_MONITOR_REPLY. */
 
     /* Nicira extensions. */
     OFPTYPE_SET_FLOW_FORMAT,      /* OFPRAW_NXT_SET_FLOW_FORMAT. */
@@ -579,10 +654,11 @@ enum ofptype {
     OFPTYPE_SET_PACKET_IN_FORMAT, /* OFPRAW_NXT_SET_PACKET_IN_FORMAT. */
     OFPTYPE_FLOW_AGE,             /* OFPRAW_NXT_FLOW_AGE. */
     OFPTYPE_SET_CONTROLLER_ID,    /* OFPRAW_NXT_SET_CONTROLLER_ID. */
+    OFPTYPE_NXT_TLV_TABLE_MOD, /* OFPRAW_NXT_TLV_TABLE_MOD. */
+    OFPTYPE_NXT_TLV_TABLE_REQUEST, /* OFPRAW_NXT_TLV_TABLE_REQUEST. */
+    OFPTYPE_NXT_TLV_TABLE_REPLY, /* OFPRAW_NXT_TLV_TABLE_REPLY. */
 
     /* Flow monitor extension. */
-    OFPTYPE_FLOW_MONITOR_STATS_REQUEST, /* OFPRAW_NXST_FLOW_MONITOR_REQUEST. */
-    OFPTYPE_FLOW_MONITOR_STATS_REPLY,   /* OFPRAW_NXST_FLOW_MONITOR_REPLY. */
     OFPTYPE_FLOW_MONITOR_CANCEL,        /* OFPRAW_NXT_FLOW_MONITOR_CANCEL. */
     OFPTYPE_FLOW_MONITOR_PAUSED,        /* OFPRAW_NXT_FLOW_MONITOR_PAUSED. */
     OFPTYPE_FLOW_MONITOR_RESUMED,       /* OFPRAW_NXT_FLOW_MONITOR_RESUMED. */
@@ -592,6 +668,9 @@ enum ofptype {
 enum ofperr ofptype_decode(enum ofptype *, const struct ofp_header *);
 enum ofperr ofptype_pull(enum ofptype *, struct ofpbuf *);
 enum ofptype ofptype_from_ofpraw(enum ofpraw);
+
+/* Information about OFTYPE_* values. */
+const char *ofptype_get_name(enum ofptype);
 
 /* OpenFlow message properties. */
 void ofpmsg_update_length(struct ofpbuf *);
@@ -622,12 +701,15 @@ bool ofpmsg_is_stat_request(const struct ofp_header *);
  * within 64 kB doesn't need any special treatment, so you might as well use
  * the ofpraw_alloc_*() functions.
  *
- * These functions work with a "struct list" of "struct ofpbuf"s, each of
+ * These functions work with a "struct ovs_list" of "struct ofpbuf"s, each of
  * which represents one part of a multipart message. */
-void ofpmp_init(struct list *, const struct ofp_header *request);
-struct ofpbuf *ofpmp_reserve(struct list *, size_t len);
-void *ofpmp_append(struct list *, size_t len);
-void ofpmp_postappend(struct list *, size_t start_ofs);
+void ofpmp_init(struct ovs_list *, const struct ofp_header *request);
+struct ofpbuf *ofpmp_reserve(struct ovs_list *, size_t len);
+void *ofpmp_append(struct ovs_list *, size_t len);
+void ofpmp_postappend(struct ovs_list *, size_t start_ofs);
+
+enum ofp_version ofpmp_version(struct ovs_list *);
+enum ofpraw ofpmp_decode_raw(struct ovs_list *);
 
 /* Decoding multipart replies. */
 uint16_t ofpmp_flags(const struct ofp_header *);
