@@ -19,29 +19,48 @@
 #include <stdlib.h>
 #include <string.h>
 #include "dynamic-string.h"
+#include "netdev-dpdk.h"
 #include "util.h"
 
 static void
-ofpbuf_use__(struct ofpbuf *b, void *base, size_t allocated,
-             enum ofpbuf_source source)
+ofpbuf_init__(struct ofpbuf *b, size_t allocated, enum ofpbuf_source source)
 {
-    b->base = b->data = base;
     b->allocated = allocated;
     b->source = source;
-    b->size = 0;
-    b->l2 = b->l2_5 = b->l3 = b->l4 = b->l7 = NULL;
+    b->header = NULL;
+    b->msg = NULL;
     list_poison(&b->list_node);
-    b->private_p = NULL;
+}
+
+static void
+ofpbuf_use__(struct ofpbuf *b, void *base, size_t allocated, size_t size,
+             enum ofpbuf_source source)
+{
+    b->base = base;
+    b->data = base;
+    b->size = size;
+
+    ofpbuf_init__(b, allocated, source);
 }
 
 /* Initializes 'b' as an empty ofpbuf that contains the 'allocated' bytes of
  * memory starting at 'base'.  'base' should be the first byte of a region
  * obtained from malloc().  It will be freed (with free()) if 'b' is resized or
  * freed. */
-void
+static void
 ofpbuf_use(struct ofpbuf *b, void *base, size_t allocated)
 {
-    ofpbuf_use__(b, base, allocated, OFPBUF_MALLOC);
+    ofpbuf_use__(b, base, allocated, 0, OFPBUF_MALLOC);
+}
+
+/* Converts ds into ofpbuf 'b'. 'b' contains the 'ds->allocated' bytes of
+ * memory starting at 'ds->string'.  'ds' should not be modified any more.
+ * The memory allocated for 'ds' will be freed (with free()) if 'b' is
+ * resized or freed. */
+void
+ofpbuf_use_ds(struct ofpbuf *b, const struct ds *ds)
+{
+    ofpbuf_use__(b, ds->string, ds->allocated + 1, ds->length, OFPBUF_MALLOC);
 }
 
 /* Initializes 'b' as an empty ofpbuf that contains the 'allocated' bytes of
@@ -61,7 +80,7 @@ ofpbuf_use(struct ofpbuf *b, void *base, size_t allocated)
 void
 ofpbuf_use_stack(struct ofpbuf *b, void *base, size_t allocated)
 {
-    ofpbuf_use__(b, base, allocated, OFPBUF_STACK);
+    ofpbuf_use__(b, base, allocated, 0, OFPBUF_STACK);
 }
 
 /* Initializes 'b' as an empty ofpbuf that contains the 'allocated' bytes of
@@ -81,7 +100,7 @@ ofpbuf_use_stack(struct ofpbuf *b, void *base, size_t allocated)
 void
 ofpbuf_use_stub(struct ofpbuf *b, void *base, size_t allocated)
 {
-    ofpbuf_use__(b, base, allocated, OFPBUF_STUB);
+    ofpbuf_use__(b, base, allocated, 0, OFPBUF_STUB);
 }
 
 /* Initializes 'b' as an ofpbuf whose data starts at 'data' and continues for
@@ -94,8 +113,7 @@ ofpbuf_use_stub(struct ofpbuf *b, void *base, size_t allocated)
 void
 ofpbuf_use_const(struct ofpbuf *b, const void *data, size_t size)
 {
-    ofpbuf_use__(b, CONST_CAST(void *, data), size, OFPBUF_STACK);
-    b->size = size;
+    ofpbuf_use__(b, CONST_CAST(void *, data), size, size, OFPBUF_STACK);
 }
 
 /* Initializes 'b' as an empty ofpbuf with an initial capacity of 'size'
@@ -110,20 +128,13 @@ ofpbuf_init(struct ofpbuf *b, size_t size)
 void
 ofpbuf_uninit(struct ofpbuf *b)
 {
-    if (b && b->source == OFPBUF_MALLOC) {
-        free(b->base);
+    if (b) {
+        if (b->source == OFPBUF_MALLOC) {
+            free(b->base);
+        }
     }
 }
 
-/* Returns a pointer that may be passed to free() to accomplish the same thing
- * as ofpbuf_uninit(b).  The return value is a null pointer if ofpbuf_uninit()
- * would not free any memory. */
-void *
-ofpbuf_get_uninit_pointer(struct ofpbuf *b)
-{
-    return b && b->source == OFPBUF_MALLOC ? b->base : NULL;
-}
-
 /* Frees memory that 'b' points to and allocates a new ofpbuf */
 void
 ofpbuf_reinit(struct ofpbuf *b, size_t size)
@@ -167,27 +178,17 @@ struct ofpbuf *
 ofpbuf_clone_with_headroom(const struct ofpbuf *buffer, size_t headroom)
 {
     struct ofpbuf *new_buffer;
-    uintptr_t data_delta;
 
-    new_buffer = ofpbuf_clone_data_with_headroom(buffer->data, buffer->size,
+    new_buffer = ofpbuf_clone_data_with_headroom(buffer->data,
+                                                 buffer->size,
                                                  headroom);
-    data_delta = (char *) new_buffer->data - (char *) buffer->data;
+    if (buffer->header) {
+        uintptr_t data_delta
+            = (char *)new_buffer->data - (char *)buffer->data;
 
-    if (buffer->l2) {
-        new_buffer->l2 = (char *) buffer->l2 + data_delta;
-    }
-    if (buffer->l2_5) {
-        new_buffer->l2_5 = (char *) buffer->l2_5 + data_delta;
-    }
-    if (buffer->l3) {
-        new_buffer->l3 = (char *) buffer->l3 + data_delta;
-    }
-    if (buffer->l4) {
-        new_buffer->l4 = (char *) buffer->l4 + data_delta;
-    }
-    if (buffer->l7) {
-        new_buffer->l7 = (char *) buffer->l7 + data_delta;
+        new_buffer->header = (char *) buffer->header + data_delta;
     }
+    new_buffer->msg = buffer->msg;
 
     return new_buffer;
 }
@@ -211,34 +212,6 @@ ofpbuf_clone_data_with_headroom(const void *data, size_t size, size_t headroom)
     return b;
 }
 
-/* Frees memory that 'b' points to, as well as 'b' itself. */
-void
-ofpbuf_delete(struct ofpbuf *b)
-{
-    if (b) {
-        ofpbuf_uninit(b);
-        free(b);
-    }
-}
-
-/* Returns the number of bytes of headroom in 'b', that is, the number of bytes
- * of unused space in ofpbuf 'b' before the data that is in use.  (Most
- * commonly, the data in a ofpbuf is at its beginning, and thus the ofpbuf's
- * headroom is 0.) */
-size_t
-ofpbuf_headroom(const struct ofpbuf *b)
-{
-    return (char*)b->data - (char*)b->base;
-}
-
-/* Returns the number of bytes that may be appended to the tail end of ofpbuf
- * 'b' before the ofpbuf must be reallocated. */
-size_t
-ofpbuf_tailroom(const struct ofpbuf *b)
-{
-    return (char*)ofpbuf_end(b) - (char*)ofpbuf_tail(b);
-}
-
 static void
 ofpbuf_copy__(struct ofpbuf *b, uint8_t *new_base,
               size_t new_headroom, size_t new_tailroom)
@@ -293,23 +266,17 @@ ofpbuf_resize__(struct ofpbuf *b, size_t new_headroom, size_t new_tailroom)
 
     new_data = (char *) new_base + new_headroom;
     if (b->data != new_data) {
-        uintptr_t data_delta = (char *) new_data - (char *) b->data;
+        if (b->header) {
+            uintptr_t data_delta = (char *) b->header - (char *) b->data;
+
+            b->header = (char *) new_data + data_delta;
+        }
+        if (b->msg) {
+            uintptr_t data_delta = (char *) b->msg - (char *) b->data;
+
+            b->msg = (char *) new_data + data_delta;
+        }
         b->data = new_data;
-        if (b->l2) {
-            b->l2 = (char *) b->l2 + data_delta;
-        }
-        if (b->l2_5) {
-            b->l2_5 = (char *) b->l2_5 + data_delta;
-        }
-        if (b->l3) {
-            b->l3 = (char *) b->l3 + data_delta;
-        }
-        if (b->l4) {
-            b->l4 = (char *) b->l4 + data_delta;
-        }
-        if (b->l7) {
-            b->l7 = (char *) b->l7 + data_delta;
-        }
     }
 }
 
@@ -335,11 +302,17 @@ ofpbuf_prealloc_headroom(struct ofpbuf *b, size_t size)
     }
 }
 
-/* Trims the size of 'b' to fit its actual content, reducing its tailroom to
- * 0.  Its headroom, if any, is preserved.
+/* Trims the size of 'b' to fit its actual content, reducing its headroom and
+ * tailroom to 0, if any.
  *
  * Buffers not obtained from malloc() are not resized, since that wouldn't save
- * any memory. */
+ * any memory.
+ *
+ * Caller needs to updates 'b->header' and 'b->msg' so that they point to the
+ * same locations in the data.  (If they pointed into the tailroom or headroom
+ * then they become invalid.)
+ *
+ */
 void
 ofpbuf_trim(struct ofpbuf *b)
 {
@@ -362,7 +335,11 @@ ofpbuf_padto(struct ofpbuf *b, size_t length)
 /* Shifts all of the data within the allocated space in 'b' by 'delta' bytes.
  * For example, a 'delta' of 1 would cause each byte of data to move one byte
  * forward (from address 'p' to 'p+1'), and a 'delta' of -1 would cause each
- * byte to move one byte backward (from 'p' to 'p-1'). */
+ * byte to move one byte backward (from 'p' to 'p-1').
+ *
+ * If used, user must make sure the 'header' and 'msg' pointers are updated
+ * after shifting.
+ */
 void
 ofpbuf_shift(struct ofpbuf *b, int delta)
 {
@@ -449,17 +426,6 @@ ofpbuf_reserve(struct ofpbuf *b, size_t size)
     b->data = (char*)b->data + size;
 }
 
-/* Reserves 'size' bytes of headroom so that they can be later allocated with
- * ofpbuf_push_uninit() without reallocating the ofpbuf. */
-void
-ofpbuf_reserve_with_tailroom(struct ofpbuf *b, size_t headroom,
-                             size_t tailroom)
-{
-    ovs_assert(!b->size);
-    ofpbuf_prealloc_tailroom(b, headroom + tailroom);
-    b->data = (char*)b->data + headroom;
-}
-
 /* Prefixes 'size' bytes to the head end of 'b', reallocating and copying its
  * data if necessary.  Returns a pointer to the first byte of the data's
  * location in the ofpbuf.  The new data is left uninitialized. */
@@ -494,67 +460,6 @@ ofpbuf_push(struct ofpbuf *b, const void *p, size_t size)
     return dst;
 }
 
-/* If 'b' contains at least 'offset + size' bytes of data, returns a pointer to
- * byte 'offset'.  Otherwise, returns a null pointer. */
-void *
-ofpbuf_at(const struct ofpbuf *b, size_t offset, size_t size)
-{
-    return offset + size <= b->size ? (char *) b->data + offset : NULL;
-}
-
-/* Returns a pointer to byte 'offset' in 'b', which must contain at least
- * 'offset + size' bytes of data. */
-void *
-ofpbuf_at_assert(const struct ofpbuf *b, size_t offset, size_t size)
-{
-    ovs_assert(offset + size <= b->size);
-    return ((char *) b->data) + offset;
-}
-
-/* Returns the byte following the last byte of data in use in 'b'. */
-void *
-ofpbuf_tail(const struct ofpbuf *b)
-{
-    return (char *) b->data + b->size;
-}
-
-/* Returns the byte following the last byte allocated for use (but not
- * necessarily in use) by 'b'. */
-void *
-ofpbuf_end(const struct ofpbuf *b)
-{
-    return (char *) b->base + b->allocated;
-}
-
-/* Clears any data from 'b'. */
-void
-ofpbuf_clear(struct ofpbuf *b)
-{
-    b->data = b->base;
-    b->size = 0;
-}
-
-/* Removes 'size' bytes from the head end of 'b', which must contain at least
- * 'size' bytes of data.  Returns the first byte of data removed. */
-void *
-ofpbuf_pull(struct ofpbuf *b, size_t size)
-{
-    void *data = b->data;
-    ovs_assert(b->size >= size);
-    b->data = (char*)b->data + size;
-    b->size -= size;
-    return data;
-}
-
-/* If 'b' has at least 'size' bytes of data, removes that many bytes from the
- * head end of 'b' and returns the first byte removed.  Otherwise, returns a
- * null pointer without modifying 'b'. */
-void *
-ofpbuf_try_pull(struct ofpbuf *b, size_t size)
-{
-    return b->size >= size ? ofpbuf_pull(b, size) : NULL;
-}
-
 /* Returns the data in 'b' as a block of malloc()'d memory and frees the buffer
  * within 'b'.  (If 'b' itself was dynamically allocated, e.g. with
  * ofpbuf_new(), then it should still be freed with, e.g., ofpbuf_delete().) */
@@ -562,6 +467,7 @@ void *
 ofpbuf_steal_data(struct ofpbuf *b)
 {
     void *p;
+
     if (b->source == OFPBUF_MALLOC && b->data == b->base) {
         p = b->data;
     } else {
@@ -570,7 +476,10 @@ ofpbuf_steal_data(struct ofpbuf *b)
             free(b->base);
         }
     }
-    b->base = b->data = NULL;
+    b->base = NULL;
+    b->data = NULL;
+    b->header = NULL;
+    b->msg = NULL;
     return p;
 }
 
@@ -582,7 +491,7 @@ ofpbuf_to_string(const struct ofpbuf *b, size_t maxbytes)
     struct ds s;
 
     ds_init(&s);
-    ds_put_format(&s, "size=%"PRIuSIZE", allocated=%"PRIuSIZE", head=%"PRIuSIZE", tail=%"PRIuSIZE"\n",
+    ds_put_format(&s, "size=%"PRIu32", allocated=%"PRIu32", head=%"PRIuSIZE", tail=%"PRIuSIZE"\n",
                   b->size, b->allocated,
                   ofpbuf_headroom(b), ofpbuf_tailroom(b));
     ds_put_hex_dump(&s, b->data, MIN(b->size, maxbytes), 0, false);
@@ -592,12 +501,11 @@ ofpbuf_to_string(const struct ofpbuf *b, size_t maxbytes)
 /* Removes each of the "struct ofpbuf"s on 'list' from the list and frees
  * them.  */
 void
-ofpbuf_list_delete(struct list *list)
+ofpbuf_list_delete(struct ovs_list *list)
 {
-    struct ofpbuf *b, *next;
+    struct ofpbuf *b;
 
-    LIST_FOR_EACH_SAFE (b, next, list_node, list) {
-        list_remove(&b->list_node);
+    LIST_FOR_EACH_POP (b, list_node, list) {
         ofpbuf_delete(b);
     }
 }
