@@ -345,28 +345,13 @@ ovsthread_wrapper(void *aux_)
 pthread_t
 ovs_thread_create(const char *name, void *(*start)(void *), void *arg)
 {
-    static struct ovsthread_once once = OVSTHREAD_ONCE_INITIALIZER;
     struct ovsthread_aux *aux;
     pthread_t thread;
     int error;
 
     forbid_forking("multiple threads exist");
     multithreaded = true;
-
-    if (ovsthread_once_start(&once)) {
-        /* The first call to this function has to happen in the main thread.
-         * Before the process becomes multithreaded we make sure that the
-         * main thread is considered non quiescent.
-         *
-         * For other threads this is done in ovs_thread_wrapper(), but the
-         * main thread has no such wrapper.
-         *
-         * There's no reason to call ovsrcu_quiesce_end() in subsequent
-         * invocations of this function and it might introduce problems
-         * for other threads. */
-        ovsrcu_quiesce_end();
-        ovsthread_once_done(&once);
-    }
+    ovsrcu_quiesce_end();
 
     aux = xmalloc(sizeof *aux);
     aux->start = start;
@@ -599,6 +584,15 @@ count_cpu_cores(void)
 
     return n_cores > 0 ? n_cores : 0;
 }
+
+/* Returns 'true' if current thread is PMD thread. */
+bool
+thread_is_pmd(void)
+{
+    const char *name = get_subprogram_name();
+    return !strncmp(name, "pmd", 3);
+}
+
 
 /* ovsthread_key. */
 
