@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011, 2012, 2013 Nicira, Inc.
+ * Copyright (c) 2011, 2012, 2013, 2015 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -67,6 +67,9 @@ bool sset_equals(const struct sset *, const struct sset *);
 struct sset_node *sset_at_position(const struct sset *,
                                    uint32_t *bucketp, uint32_t *offsetp);
 
+/* Set operations. */
+void sset_intersect(struct sset *, const struct sset *);
+
 /* Iteration macros. */
 #define SSET_FOR_EACH(NAME, SSET)               \
     for ((NAME) = SSET_FIRST(SSET);             \
@@ -80,6 +83,7 @@ struct sset_node *sset_at_position(const struct sset *,
           : false);                                 \
          (NAME) = (NEXT))
 
+const char **sset_array(const struct sset *);
 const char **sset_sort(const struct sset *);
 
 /* Implementation helper macros. */
