@@ -74,7 +74,7 @@ int get_cpu_usage(void);
 
 long long int time_boot_msec(void);
 
-void timewarp_wait(void);
+void timewarp_run(void);
 
 #ifdef  __cplusplus
 }
