@@ -29,8 +29,8 @@
 #include "stream.h"
 #include "util.h"
 #include "vconn-provider.h"
-#include "vconn.h"
-#include "vlog.h"
+#include "openvswitch/vconn.h"
+#include "openvswitch/vlog.h"
 
 VLOG_DEFINE_THIS_MODULE(vconn_stream);
 
@@ -78,7 +78,7 @@ vconn_stream_open(const char *name, uint32_t allowed_versions,
     struct stream *stream;
     int error;
 
-    error = stream_open_with_default_port(name, OFP_OLD_PORT, &stream, dscp);
+    error = stream_open_with_default_port(name, OFP_PORT, &stream, dscp);
     if (!error) {
         error = stream_connect(stream);
         if (!error || error == EAGAIN) {
@@ -311,8 +311,7 @@ pvconn_pstream_listen(const char *name, uint32_t allowed_versions,
     struct pstream *pstream;
     int error;
 
-    error = pstream_open_with_default_port(name, OFP_OLD_PORT,
-                                           &pstream, dscp);
+    error = pstream_open_with_default_port(name, OFP_PORT, &pstream, dscp);
     if (error) {
         return error;
     }
