@@ -1,4 +1,4 @@
-/* Copyright (c) 2009, 2010, 2011, 2012, 2013 Nicira, Inc.
+/* Copyright (c) 2009, 2010, 2011, 2012, 2013, 2014, 2015 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,17 @@
 #ifndef OFPROTO_DPIF_XLATE_H
 #define OFPROTO_DPIF_XLATE_H 1
 
+#include "dp-packet.h"
 #include "flow.h"
 #include "meta-flow.h"
 #include "odp-util.h"
 #include "ofpbuf.h"
 #include "ofproto-dpif-mirror.h"
+#include "ofproto-dpif-rid.h"
 #include "ofproto-dpif.h"
 #include "ofproto.h"
 #include "stp.h"
+#include "ovs-lldp.h"
 
 struct bfd;
 struct bond;
@@ -31,34 +34,15 @@ struct lacp;
 struct dpif_ipfix;
 struct dpif_sflow;
 struct mac_learning;
+struct mcast_snooping;
 struct xlate_cache;
 
-struct xlate_recirc {
-    uint32_t recirc_id;  /* !0 Use recirculation instead of output. */
-    uint8_t  hash_alg;   /* !0 Compute hash for recirc before. */
-    uint32_t hash_basis;  /* Compute hash for recirc before. */
-};
-
 struct xlate_out {
-    /* Wildcards relevant in translation.  Any fields that were used to
-     * calculate the action must be set for caching and kernel
-     * wildcarding to work.  For example, if the flow lookup involved
-     * performing the "normal" action on IPv4 and ARP packets, 'wc'
-     * would have the 'in_port' (always set), 'dl_type' (flow match),
-     * 'vlan_tci' (normal action), and 'dl_dst' (normal action) fields
-     * set. */
-    struct flow_wildcards wc;
-
     enum slow_path_reason slow; /* 0 if fast path may be used. */
     bool fail_open;             /* Initial rule is fail open? */
-    bool has_learn;             /* Actions include NXAST_LEARN? */
-    bool has_normal;            /* Actions output to OFPP_NORMAL? */
-    bool has_fin_timeout;       /* Actions include NXAST_FIN_TIMEOUT? */
-    ofp_port_t nf_output_iface; /* Output interface index for NetFlow. */
-    mirror_mask_t mirrors;      /* Bitmap of associated mirrors. */
 
-    uint64_t odp_actions_stub[256 / 8];
-    struct ofpbuf odp_actions;
+    struct recirc_refs recircs; /* Recirc action IDs on which references are
+                                 * held. */
 };
 
 struct xlate_in {
@@ -70,7 +54,7 @@ struct xlate_in {
 
     /* The packet corresponding to 'flow', or a null pointer if we are
      * revalidating without a packet to refer to. */
-    const struct ofpbuf *packet;
+    const struct dp_packet *packet;
 
     /* Should OFPP_NORMAL update the MAC learning table?  Should "learn"
      * actions update the flow table?
@@ -80,12 +64,6 @@ struct xlate_in {
      * not if we are just revalidating. */
     bool may_learn;
 
-    /* If the caller of xlate_actions() doesn't need the flow_wildcards
-     * contained in struct xlate_out.  'skip_wildcards' can be set to true
-     * disabling the expensive wildcard computation.  When true, 'wc' in struct
-     * xlate_out is undefined and should not be read. */
-    bool skip_wildcards;
-
     /* The rule initiating translation or NULL. If both 'rule' and 'ofpacts'
      * are NULL, xlate_actions() will do the initial rule lookup itself. */
     struct rule_dpif *rule;
@@ -117,7 +95,8 @@ struct xlate_in {
      * significant decision, e.g. to explain why OFPP_NORMAL translation
      * dropped a packet.  'recurse' is the resubmit recursion depth at time of
      * invocation. */
-    void (*report_hook)(struct xlate_in *, const char *s, int recurse);
+    void (*report_hook)(struct xlate_in *, int recurse,
+                        const char *format, va_list args);
 
     /* If nonnull, flow translation credits the specified statistics to each
      * rule reached through a resubmit or OFPP_TABLE action.
@@ -126,6 +105,20 @@ struct xlate_in {
      * calling xlate_in_init(). */
     const struct dpif_flow_stats *resubmit_stats;
 
+    /* Recursion and resubmission levels carried over from a pre-existing
+     * translation of a related flow. An example of when this can occur is
+     * the translation of an ARP packet that was generated as the result of
+     * outputting to a tunnel port. In this case, the original flow going to
+     * the tunnel is the related flow. Since the two flows are different, they
+     * should not use the same xlate_ctx structure. However, we still need
+     * limit the maximum recursion across the entire translation.
+     *
+     * These fields are normally set to zero, so the client has to set them
+     * manually after calling xlate_in_init(). In that case, they should be
+     * copied from the same-named fields in the related flow's xlate_ctx. */
+    int recurse;
+    int resubmits;
+
     /* If nonnull, flow translation populates this cache with references to all
      * modules that are affected by translation. This 'xlate_cache' may be
      * passed to xlate_push_stats() to perform the same function as
@@ -134,62 +127,90 @@ struct xlate_in {
      * This is normally null so the client has to set it manually after
      * calling xlate_in_init(). */
     struct xlate_cache *xcache;
+
+    /* If nonnull, flow translation puts the resulting datapath actions in this
+     * buffer.  If null, flow translation will not produce datapath actions. */
+    struct ofpbuf *odp_actions;
+
+    /* If nonnull, flow translation populates this with wildcards relevant in
+     * translation.  Any fields that were used to calculate the action are set,
+     * to allow caching and kernel wildcarding to work.  For example, if the
+     * flow lookup involved performing the "normal" action on IPv4 and ARP
+     * packets, 'wc' would have the 'in_port' (always set), 'dl_type' (flow
+     * match), 'vlan_tci' (normal action), and 'dl_dst' (normal action) fields
+     * set. */
+    struct flow_wildcards *wc;
+
+    /* The recirculation context related to this translation, as returned by
+     * xlate_lookup. */
+    const struct recirc_id_node *recirc;
 };
 
-extern struct fat_rwlock xlate_rwlock;
-
-void xlate_ofproto_set(struct ofproto_dpif *, const char *name,
-                       struct dpif *, struct rule_dpif *miss_rule,
-                       struct rule_dpif *no_packet_in_rule,
+void xlate_ofproto_set(struct ofproto_dpif *, const char *name, struct dpif *,
                        const struct mac_learning *, struct stp *,
+                       struct rstp *, const struct mcast_snooping *,
                        const struct mbridge *, const struct dpif_sflow *,
                        const struct dpif_ipfix *, const struct netflow *,
-                       enum ofp_config_flags, bool forward_bpdu,
-                       bool has_in_band, bool enable_recirc,
-                       bool variable_length_userdata,
-                       size_t mpls_label_stack_length)
-    OVS_REQ_WRLOCK(xlate_rwlock);
-void xlate_remove_ofproto(struct ofproto_dpif *) OVS_REQ_WRLOCK(xlate_rwlock);
+                       bool forward_bpdu, bool has_in_band,
+                       const struct dpif_backer_support *support);
+void xlate_remove_ofproto(struct ofproto_dpif *);
 
 void xlate_bundle_set(struct ofproto_dpif *, struct ofbundle *,
                       const char *name, enum port_vlan_mode, int vlan,
                       unsigned long *trunks, bool use_priority_tags,
                       const struct bond *, const struct lacp *,
-                      bool floodable) OVS_REQ_WRLOCK(xlate_rwlock);
-void xlate_bundle_remove(struct ofbundle *) OVS_REQ_WRLOCK(xlate_rwlock);
+                      bool floodable);
+void xlate_bundle_remove(struct ofbundle *);
 
 void xlate_ofport_set(struct ofproto_dpif *, struct ofbundle *,
                       struct ofport_dpif *, ofp_port_t, odp_port_t,
-                      const struct netdev *, const struct cfm *,
-                      const struct bfd *, struct ofport_dpif *peer,
-                      int stp_port_no, const struct ofproto_port_queue *qdscp,
+                      const struct netdev *, const struct cfm *, const struct bfd *,
+                      const struct lldp *, struct ofport_dpif *peer,
+                      int stp_port_no, const struct rstp_port *rstp_port,
+                      const struct ofproto_port_queue *qdscp,
                       size_t n_qdscp, enum ofputil_port_config,
                       enum ofputil_port_state, bool is_tunnel,
-                      bool may_enable) OVS_REQ_WRLOCK(xlate_rwlock);
-void xlate_ofport_remove(struct ofport_dpif *) OVS_REQ_WRLOCK(xlate_rwlock);
+                      bool may_enable);
+void xlate_ofport_remove(struct ofport_dpif *);
 
-int xlate_receive(const struct dpif_backer *, struct ofpbuf *packet,
-                  const struct nlattr *key, size_t key_len,
-                  struct flow *, struct ofproto_dpif **, struct dpif_ipfix **,
-                  struct dpif_sflow **, struct netflow **,
-                  odp_port_t *odp_in_port)
-    OVS_EXCLUDED(xlate_rwlock);
+struct ofproto_dpif * xlate_lookup_ofproto(const struct dpif_backer *,
+                                           const struct flow *,
+                                           ofp_port_t *ofp_in_port);
+int xlate_lookup(const struct dpif_backer *, const struct flow *,
+                 struct ofproto_dpif **, struct dpif_ipfix **,
+                 struct dpif_sflow **, struct netflow **,
+                 ofp_port_t *ofp_in_port);
+
+enum xlate_error {
+    XLATE_OK = 0,
+    XLATE_BRIDGE_NOT_FOUND,
+    XLATE_RECURSION_TOO_DEEP,
+    XLATE_TOO_MANY_RESUBMITS,
+    XLATE_STACK_TOO_DEEP,
+    XLATE_NO_RECIRCULATION_CONTEXT,
+    XLATE_RECIRCULATION_CONFLICT,
+    XLATE_TOO_MANY_MPLS_LABELS,
+};
+
+const char *xlate_strerror(enum xlate_error error);
+
+enum xlate_error xlate_actions(struct xlate_in *, struct xlate_out *);
 
-void xlate_actions(struct xlate_in *, struct xlate_out *)
-    OVS_EXCLUDED(xlate_rwlock);
 void xlate_in_init(struct xlate_in *, struct ofproto_dpif *,
-                   const struct flow *, struct rule_dpif *,
-                   uint16_t tcp_flags, const struct ofpbuf *packet);
+                   const struct flow *, ofp_port_t in_port, struct rule_dpif *,
+                   uint16_t tcp_flags, const struct dp_packet *packet,
+                   struct flow_wildcards *, struct ofpbuf *odp_actions);
 void xlate_out_uninit(struct xlate_out *);
 void xlate_actions_for_side_effects(struct xlate_in *);
-void xlate_out_copy(struct xlate_out *dst, const struct xlate_out *src);
 
-int xlate_send_packet(const struct ofport_dpif *, struct ofpbuf *);
+int xlate_send_packet(const struct ofport_dpif *, struct dp_packet *);
 
 struct xlate_cache *xlate_cache_new(void);
-void xlate_push_stats(struct xlate_cache *, bool may_learn,
-                      const struct dpif_flow_stats *);
+void xlate_push_stats(struct xlate_cache *, const struct dpif_flow_stats *);
 void xlate_cache_clear(struct xlate_cache *);
 void xlate_cache_delete(struct xlate_cache *);
 
+void xlate_txn_start(void);
+void xlate_txn_commit(void);
+
 #endif /* ofproto-dpif-xlate.h */
