@@ -35,7 +35,7 @@
 #include "transaction.h"
 #include "uuid.h"
 #include "util.h"
-#include "vlog.h"
+#include "openvswitch/vlog.h"
 
 VLOG_DEFINE_THIS_MODULE(ovsdb_file);
 
