@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009, 2010, 2013 Nicira, Inc.
+ * Copyright (c) 2009, 2010, 2014 Nicira, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,14 +15,13 @@
  */
 
 #include <config.h>
-
+#undef NDEBUG
 #include "json.h"
-
 #include <ctype.h>
 #include <errno.h>
 #include <getopt.h>
 #include <stdio.h>
-
+#include "ovstest.h"
 #include "util.h"
 
 /* --pretty: If set, the JSON output is pretty-printed, instead of printed as
@@ -42,7 +41,6 @@ print_and_free_json(struct json *json)
         ok = false;
     } else {
         char *s = json_to_string(json, JSSF_SORT | (pretty ? JSSF_PRETTY : 0));
-        ovs_assert(pretty || json_serialized_length(json) == strlen(s));
         puts(s);
         free(s);
         ok = true;
@@ -104,8 +102,8 @@ parse_multiple(FILE *stream)
     return ok;
 }
 
-int
-main(int argc, char *argv[])
+static void
+test_json_main(int argc, char *argv[])
 {
     const char *input_file;
     FILE *stream;
@@ -155,5 +153,7 @@ main(int argc, char *argv[])
 
     fclose(stream);
 
-    return !ok;
+    exit(!ok);
 }
+
+OVSTEST_REGISTER("test-json", test_json_main);
